/*
 * CrudContainer
 * Copyright (C) 2011 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.crud;

/**
 * Interface for listeners of Crud actions.
 */
public interface CrudListener<IDT, BT> {

    /**
     * Called before an entity is read.
     */
    void onBeforeEntityRead(CrudEvent.WithId<IDT, BT> event);

    /**
     * Called when an entity is read.
     */
    void onEntityRead(CrudEvent.WithEntity<IDT, BT> event);

    /**
     * Called before an entity is deleted.
     */
    void onBeforeEntityDelete(CrudEvent.WithId<IDT, BT> event);

    /**
     * Called when an entity is deleted.
     * @param event
     */
    void onEntityDelete(CrudEvent.WithId<IDT, BT> event);

    /**
     * Called before an entity is being persisted.
     */
    void onBeforeEntityUpdate(CrudEvent.WithEntity<IDT, BT> event);

    /**
     * Called when an entity is persisted.
     * @param event
     */
    void onEntityUpdate(CrudEvent.WithEntity<IDT, BT> event);

    /**
     * Called before an entity is created.
     */
    void onBeforeEntityCreate(CrudEvent<IDT, BT> event);

    /**
     * Called when an entity is created.
     */
    void onEntityCreate(CrudEvent.WithEntity<IDT, BT> event);
}

